package net.sourceforge.webstretch.request;

import java.net.URLDecoder;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
 * Records all information in a request body
 * @author Simon Shanks
 */
public class RequestBody {
	
	private final static char PARAMETER_VALUE_SEPERATOR='=';
	private final static String PARAMETER_SEPERATOR="&";
	
	private HashMap parameters = new HashMap();
	
	public RequestBody(byte[] body){
		// convert the body to a string so that the information can be extracted
		String completeRequest = RequestBody.convertBodyToString(body);
		StringTokenizer st =
			new StringTokenizer(completeRequest,PARAMETER_SEPERATOR);
		while (st.hasMoreTokens()) {
			String valuePair = st.nextToken();
			int positionOfEqualsSign = valuePair.indexOf(PARAMETER_VALUE_SEPERATOR);
			String elementName = valuePair.substring(0, positionOfEqualsSign);
			String elementValue =
				URLDecoder
					.decode(
						valuePair.substring(
							positionOfEqualsSign + 1,
							valuePair.length()))
					.trim();
			parameters.put(elementName,elementValue);
		}			
	}
	
	/**
	 * Converts the byte array of a body to a string for textual manipulation.
	 * @param body
	 * @return Returns an empty string is no body is present. Otherwise, the body
	 * in String representation is returned.
	 */
	public static String convertBodyToString(byte[] body){
		if ((body==null)||body.length==0)
			return "";
		// convert the body to a string so that the information can be extracted
		StringBuffer completeRequest = new StringBuffer(body.length);
		for (int i = 0; i < body.length; i++) {
			completeRequest.append((char) body[i]);
		}
		return completeRequest.toString();
	}
	
	/**
	 * Gets a list of parameter names, provided in the body as part of a POST
	 * from a submitted form
	 * @return
	 */
	public String[] getParameterNames(){
		Collection parameterCollection = parameters.keySet();
		String[] parameterNames = new String[parameterCollection.size()];
		Iterator iterator = parameterCollection.iterator();
		int i = 0;
		while (iterator.hasNext()) {
			parameterNames[i] = (String) iterator.next();
			i++;
		}
		return parameterNames;
	}
	
	/**
	 * Get a value of a parameter, which can be provideed in the body as part of a POST
	 * from a submitted form
	 * @param parameterName
	 * @return
	 */
	public String getParameterValue(String parameterName){
		return (String)parameters.get(parameterName);
	}

}
