package net.sourceforge.webstretch.request;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

/**
 * Records the information in a request header
 * @author Simon Shanks
 */
public class RequestHeaders {
	
	public static final char HEADER_VALUE_SEPERATOR=':';
	public static final String COOKIE_PARAM_NAME="Cookie";
	
	public static boolean cookiesAllowed=true;
	public static String defaultUserAgent;
	
	private static final String USER_AGENT="User-Agent";
	private HashMap headers = new HashMap();

	public RequestHeaders() {
	}
	
	public RequestHeaders(ArrayList requestHeaders) {
		for (int i = 0; i < requestHeaders.size(); i++) {
			this.processHeaderLine((String)requestHeaders.get(i));
		}
	}
	
	/**
	 * Strips information from a single header line
	 * @param headerLine A single line from an entity header
	 */
	private void processHeaderLine(String headerLine) {
		int positionOfSeperator = headerLine.indexOf(HEADER_VALUE_SEPERATOR);
		String headerName = headerLine.substring(0, positionOfSeperator);
		String headerValue =
			headerLine.substring(positionOfSeperator + 2, headerLine.length());
		this.setHeader(headerName, headerValue);
	}
	
	/**
	 * Sets a header entity pair. Any old values with the same name will be replaced.
	 * @param header The name of the header to store.
	 * @param value The value to associate with the header name.
	 */
	public void setHeader(String header, String value) {
		if (!cookiesAllowed && header.equalsIgnoreCase(COOKIE_PARAM_NAME)) 
			return;
		if (header.equalsIgnoreCase(USER_AGENT)){
			if (defaultUserAgent==null)
				defaultUserAgent=value;
			else
				value=defaultUserAgent;
		}
		headers.put(header, value);
	}
	
	/**
	 * Returns a list of all header entities in the request. 
	 * @return All of the header names.
	 */
	public String[] getHeaderNames() {
		Collection headerCollection = headers.keySet();
		String[] headerNames = new String[headerCollection.size()];
		Iterator iterator = headerCollection.iterator();
		int i = 0;
		while (iterator.hasNext()) {
			headerNames[i] = (String) iterator.next();
			i++;
		}
		return headerNames;
	}

	
	/**
	 * Gets the value(s) associated with a header which has been set
	 * @param headerName The name of the header entity to look up.
	 * @return The value of the header entity.
	 */
	public String getHeaderValue(String headerName) {
		return (String) headers.get(headerName);
	}
	
	public String toString(){
		StringBuffer headerRequest=new StringBuffer();
		String[] headerNames=this.getHeaderNames();
		for (int i = 0; i < headerNames.length; i++) {
			// write out header
			String headerName=headerNames[i];
			String headerValue=this.getHeaderValue(headerName);
			headerRequest.append(headerName);
			headerRequest.append(String.valueOf(RequestHeaders.HEADER_VALUE_SEPERATOR));
			headerRequest.append(headerValue);
			headerRequest.append("\r\n");
		}	
		return headerRequest.toString();
	}

}
