package net.sourceforge.webstretch.request;

import java.io.*;
import java.util.*;

import net.sourceforge.webstretch.server.AlertWords;

/**
 * Extracts information from the incoming input stream of the web request
 * @author Simon Shanks
 */
public class RequestPayload {

	private static final char BODY_END_POSITION='\0';
	
	private String requestLine;
	private RequestHeaders requestHeaders;
	private byte[] body;
	private boolean containsAlertWord=false;

	public RequestPayload(BufferedInputStream in) throws IOException {
		ArrayList requestData = new ArrayList();
		ArrayList requestHeaderList=new ArrayList();
		// TODO fix for requests greater than buffer size or make it more memory efficent
		byte[] buffer = new byte[250000];
		int numRead = 0;

		// process request coming from client
		do {
			numRead = in.read(buffer);
			int markedPosition = 0;
			String currentLine = null;
			for (int i = 0; i < numRead - 1; i++) {
				if ((char) buffer[i] == '\r' && (char) buffer[i + 1] == '\n') {
					if (markedPosition == i) {
						//found blank line
						int bodyStartPosition = i + 2;
						int bodyEndPosition = bodyStartPosition;
						while ((char) buffer[bodyEndPosition] != BODY_END_POSITION) {
							// found body
							bodyEndPosition++;
						}
						if (bodyStartPosition != bodyEndPosition) {
							// deal with body
							int bodyLength =
								bodyEndPosition - bodyStartPosition;
							body = new byte[bodyLength];
							for (int j = 0; j < bodyLength; j++) {
								body[j] = buffer[bodyStartPosition + j];
							}
							// body now populated
							
							// test for AlertWords
							containsAlertWord=AlertWords.containsAlertWord(body);
						}
						break;
					}
					currentLine =
						new String(buffer, markedPosition, i - markedPosition);
					markedPosition = i + 2;
					if (requestLine==null)	
						requestLine=currentLine;
					else
						requestHeaderList.add(currentLine);
						
					// test for AlertWords
					if (containsAlertWord==false)
						containsAlertWord=AlertWords.containsAlertWord(currentLine);
				}
			}
		}
		while ((numRead == buffer.length) && (numRead != -1));
		
		requestHeaders=new RequestHeaders(requestHeaderList);
	}
	
	public boolean hasBody(){
		return (body!=null);
	}

	/**
	 * @return Returns null if no body in request
	 */
	public byte[] getBody() {
		return body;
	}
	
	public String getRequestLine(){
		return requestLine;
	}
	
	public boolean containsAlertWord(){
		return containsAlertWord;
	}

	/**
	 * @return Returns null if no headers in request
	 */
	public RequestHeaders getRequestHeaders() {
		return requestHeaders;
	}

}
