package net.sourceforge.webstretch.response;

import java.io.*;

/**
 * Standard methods which are required in a web response
 * @author Simon Shanks
 */
public abstract class AbstractResponse {
	
	protected static final String CONTENT_TYPE="Content-Type: ";
	protected static final String CONTENT_LENGTH="Content-Length: ";
	protected static final String CRLF="\r\n";
	
	private static final String MIME_TYPE_HTML="text/html";
	private static final String MIME_TYPE_CSS="text/css";
	private static final String MIME_TYPE_GIF="image/gif";
	
	/**
	 * A HTTP/1.1 200 OK response
	 * @return A string value which may be passed to the client in a response.
	 */
	protected String getOKResponse(){
		return "HTTP/1.1 200 OK"+CRLF;
	}
	
	/**
	 * The pragma header instructing the client no to cache data
	 * @return A string value which may be passed to the client in a response. 
	 */
	protected String getNoCacheHeader(){
		return "Pragma: no-cache"+CRLF;
	}
	
	protected String getHeaderResponse(long length){
		StringBuffer headerContent=new StringBuffer(this.getOKResponse());
		headerContent.append(CONTENT_TYPE);
		headerContent.append(MIME_TYPE_HTML);
		headerContent.append(CRLF);
		headerContent.append(CONTENT_LENGTH+length);
		headerContent.append(this.getNoCacheHeader());
		headerContent.append("Connection: close");
		headerContent.append(CRLF);
		return headerContent.toString();
	}
	
	/**
	 * Contructs the content type for a local file
	 * @param localFile A file on the proxy servers filesystem
	 * @return The header value which can be used in a web response
	 */
	protected String getContentType(File localFile) {
		String fileName = localFile.getName();
		int positionOfExtension = fileName.lastIndexOf('.');
		String extension =
				fileName.substring(positionOfExtension, fileName.length());
		StringBuffer contentResponse=new StringBuffer(CONTENT_TYPE);
		if (extension.equalsIgnoreCase(".html")) {
			contentResponse.append(MIME_TYPE_HTML);
		} else
		if (extension.equalsIgnoreCase(".gif")) {
			contentResponse.append(MIME_TYPE_GIF);
		} else
		if (extension.equalsIgnoreCase(".css")) {
			contentResponse.append(MIME_TYPE_CSS);
		}
		contentResponse.append(CRLF);
		return contentResponse.toString();
	}
	
	/**
	 * Sends CRLF to the client
	 */
	protected void closeHeader(BufferedOutputStream out) throws IOException {
		out.write(CRLF.getBytes());
		out.flush();
	}
	
	/**
	 * Sends a response for a web request to the client
	 * @param out The output stream to the client
	 * @throws Exception
	 */
	public abstract void sendResultsToClient(BufferedOutputStream out) throws Exception;

}
