package net.sourceforge.webstretch.response;

import java.io.BufferedOutputStream;

import net.sourceforge.webstretch.request.RequestBody;
import net.sourceforge.webstretch.request.RequestHeaders;
import net.sourceforge.webstretch.server.AlertWords;
import net.sourceforge.webstretch.server.InspectionDecision;

/**
 * Allows the user to configure the proxies current settings
 * @author Simon Shanks
 */
public class ConfigRequestResponse extends AbstractHTMLResponse {
	
	private static String INPUT_INSPECTION_NAME="inspection";
	private static String INPUT_COOKIES_NAME="cookies";
	private static String INPUT_SEARCH_STRINGS="searchstrings";
	private static String INPUT_BROWSER_AGENT="browseragent";
	
	private static String CHECKBOX_CHECKED="on";
	
	private static String[][] BROWSER_AGENTS = new String[][]{
		{"Dont Change",""},
// 		TODO let settings return to original browser setting
//		{"Original Setting","original"},
		{"========Browsers=============",""},
		{"Lynx","Lynx/2.6 libwww-FM/2.14"},
		{"Mozilla","Mozilla/5.0 (Windows; U; Win 9x 4.90) Gecko/20020502 CS 2000 7.0/7.0"},
		{"MS IE6.0","Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)"},
		{"MS IE5.5","Mozilla/4.0 (compatible; MSIE 5.5; Windows 95)"},
		{"Netscape 7","Mozilla/5.0 (X11; U; SunOS sun4u; en-US; rv:1.0.1) Gecko/20020921 Netscape/7.0"},
		{"Opera 7.5","Opera/7.50 (X11; Linux i686; U) [en]"},		
		{"Safari 1","Mozilla/5.0 (Macintosh; U; PPC Mac OS X; en) AppleWebKit/106.2 (KHTML, like Gecko) Safari/100.1"},
		{"========Mobile Devices=======",""},
		{"AvantGo 3.2","Mozilla/3.0 (compatible; AvantGo 3.2)"},
		{"BlackBerry 7330/3.7.0","BlackBerry7730/3.7.0"},
		{"Nokia 7250 Phone","Nokia7250/1.0 (3.14) Profile/MIDP-1.0 Configuration/CLDC-1.0"},
		{"Pocket PC","Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; PPC; 240x320)"},
		{"SonyEricsson T610 Phone","SonyEricssonT610/R201 Profile/MIDP-1.0 Configuration/CLDC-1.0"},
		{"========Search Engines=======",""},				
		{"Google Bot","Googlebot/2.1 (+http://www.googlebot.com/bot.html)"},
		{"Zyborg","Mozilla/4.0 compatible ZyBorg/1.0 (ZyBorg@WISEnutbot.com; http://www.WISEnutbot.com)"},
		};
	
	private boolean changesApplied=false;
	
	/**
	 * 
	 * @param body The body containing form input which may be provided by the forms POST method
	 */
	public ConfigRequestResponse(byte[] body){
		if (body==null){
			// form on page hasnt been submitted
			changesApplied=false;
			return;	
		}	
		RequestBody requestBody=new RequestBody(body);
		{
			// test whether to switch inspection filtering
			String inspectionValue=requestBody.getParameterValue(INPUT_INSPECTION_NAME);
			if (inspectionValue==null)
				InspectionDecision.inspectionActivated=false;
			else if (inspectionValue.equals(CHECKBOX_CHECKED))
				InspectionDecision.inspectionActivated=true;	
		}
		{
			// test whether to switch cookie filtering
			String cookieValue=requestBody.getParameterValue(INPUT_COOKIES_NAME);
			if (cookieValue==null)
				RequestHeaders.cookiesAllowed=false;
			else if (cookieValue.equals(CHECKBOX_CHECKED))
				RequestHeaders.cookiesAllowed=true;
		}
		{
			// test whether alert words have been set
			String alertWords=requestBody.getParameterValue(INPUT_SEARCH_STRINGS);
			if (alertWords!=null && alertWords.length()>0)
				AlertWords.setWords(alertWords);
			else
				AlertWords.reset();
		}
		{
			// test whether to change default user agent
			String userAgentValue=requestBody.getParameterValue(INPUT_BROWSER_AGENT);
			if (userAgentValue!=null && userAgentValue.length()>0)
//				if (userAgentValue.compareTo("original")==0)
//					RequestHeaders.defaultUserAgent=null;
//				else
					RequestHeaders.defaultUserAgent=userAgentValue;
		}

		changesApplied=true;
	}

	/* (non-Javadoc)
	 * @see net.sourceforge.webstretch.response.ProxyResponse#sendResultsToClient(java.io.BufferedOutputStream)
	 */
	public void sendResultsToClient(BufferedOutputStream out)
		throws Exception {
			StringBuffer htmlContent = new StringBuffer();
			htmlContent.append(getStartOfHTML("WebStretch Configuration"));
			
			htmlContent.append("<table class=\"headertable\"><tr>");
			htmlContent.append("<td class=\"header\" align=\"center\"><a href=\"http://webstretch/configserver_action\">");
			htmlContent.append("Server Configuration");
			htmlContent.append("</a></td><td class=\"header\" align=\"center\">Request Configuration</td><td style=\"background:white;\"></td></tr></table>");
			
			htmlContent.append("<h2>Proxy Request Settings</h2>");
			
			htmlContent.append("<form action=\"/configrequest_action\" method=\"post\">");
			
			htmlContent.append("<input type=\"checkbox\" name=\""+INPUT_INSPECTION_NAME+"\" ");
			if (InspectionDecision.inspectionActivated)
				htmlContent.append("checked ");
			htmlContent.append("style=\"width:20;\"> Inspect all HTML requests");
			htmlContent.append("<input type=\"checkbox\" name=\""+INPUT_COOKIES_NAME+"\" ");
			if (RequestHeaders.cookiesAllowed)
				htmlContent.append("checked ");
			htmlContent.append("style=\"width:20;\"> Allow cookies to be sent");
			
			htmlContent.append("<p class=\"dashed\">");
			htmlContent.append("<h2>Browser Agent</h2>");
			htmlContent.append("Current User Agent (browser)<br/>");
			htmlContent.append(RequestHeaders.defaultUserAgent);
			htmlContent.append("</p><p>Alter Default User Agent (browser)&nbsp;");
			htmlContent.append("<select name=\""+INPUT_BROWSER_AGENT+"\">");
			for (int i=0;i<BROWSER_AGENTS.length;i++){
				htmlContent.append("<option value=\"");
				htmlContent.append(BROWSER_AGENTS[i][1]);
				htmlContent.append("\">");
				htmlContent.append(BROWSER_AGENTS[i][0]);
				htmlContent.append("</option>");
			}
			htmlContent.append("</select>");
			htmlContent.append("</p>");
			
			htmlContent.append("<p class=\"dashed\">");
			htmlContent.append("<h2>Alert Words</h2>");
			htmlContent.append("Alert user if any of the following strings are present in a request<br/>(if multiple words, use a comma to seperate them)...");
			htmlContent.append("</p>");
			
			htmlContent.append("<input type=\"text\" name=\""+INPUT_SEARCH_STRINGS+"\" value=\""+AlertWords.getWords()+"\">");

			htmlContent.append("<p>");
			htmlContent.append("<i>The request will be intercepted in the browser if a word is detected, even if request interception is turned off.</i>");
			htmlContent.append("</p>");			
			
			htmlContent.append("<p><input type=\"submit\" value=\"Apply Changes\"></p>");
			if (changesApplied)
				htmlContent.append("<p><font color=\"red\">Changes Applied!</font></p>");
			htmlContent.append("</form>");
			htmlContent.append(getEndOfHTML());
			
			{
				// send header details		
				String headerContent=getHeaderResponse(htmlContent.toString().getBytes().length);
				out.write(headerContent.getBytes());
			}
			this.closeHeader(out);
			
			// send body
			out.write(htmlContent.toString().getBytes());
	}

}
