package net.sourceforge.webstretch.response;

import java.io.BufferedOutputStream;

import net.sourceforge.webstretch.request.RequestBody;
import net.sourceforge.webstretch.server.ProxyConfiguration;

/**
 * Allows the user to configure the proxies current settings
 * @author Simon Shanks
 */
public class ConfigServerResponse extends AbstractHTMLResponse {
	
	private static String EXT_PROXY_HOST="externalproxyhost";
	private static String EXT_PROXY_PORT="externalproxyport";
	private static String IP_RESTRICTION="iprestriction";
	
	private boolean changesApplied=false;
	
	/**
	 * 
	 * @param body The body containing form input which may be provided by the forms POST method
	 */
	public ConfigServerResponse(byte[] body){
		if (body==null){
			// form on page hasnt been submitted
			changesApplied=false;
			return;	
		}	
		RequestBody requestBody=new RequestBody(body);
		{
			// test whether to change external proxy
			String extProxyHost=requestBody.getParameterValue(EXT_PROXY_HOST);
			if (extProxyHost!=null && extProxyHost.length()>0){
				ProxyConfiguration.setExternalProxyHost(extProxyHost);
				String extProxyPort=requestBody.getParameterValue(EXT_PROXY_PORT);
				if (extProxyPort!=null && extProxyPort.length()>0){
					try {
						int proxyPort=Integer.parseInt(extProxyPort);
						ProxyConfiguration.setExternalProxyPort(proxyPort);
					} catch (NumberFormatException e){
						// dont bother updating anything
					}					
				}	
			}			
		}
		{
			String restrictedIP=requestBody.getParameterValue(IP_RESTRICTION);
			if (restrictedIP!=null && restrictedIP.length()>0){
				ProxyConfiguration.setRestrictIPAddress(restrictedIP);
			}
		}
		changesApplied=true;
	}

	/* (non-Javadoc)
	 * @see net.sourceforge.webstretch.response.ProxyResponse#sendResultsToClient(java.io.BufferedOutputStream)
	 */
	public void sendResultsToClient(BufferedOutputStream out)
		throws Exception {
			StringBuffer htmlContent = new StringBuffer();
			htmlContent.append(getStartOfHTML("WebStretch Configuration"));
			
			htmlContent.append("<table class=\"headertable\"><tr>");
			htmlContent.append("<td class=\"header\" align=\"center\">");
			htmlContent.append("Server Configuration");
			htmlContent.append("</td><td class=\"header\" align=\"center\"><a href=\"http://webstretch/configrequest_action\">Request Configuration</a></td><td style=\"background:white;\"></td></tr></table>");
			
			htmlContent.append("<h2>External Proxy (Proxy Chaining)</h2>");
			
			htmlContent.append("<form action=\"/configserver_action\" method=\"post\">");	
			htmlContent.append("<p>If you usually use a web proxy server to access web sites, please enter the proxy settings here. <i>Do not enter your Webstretch proxy settings.</i></p>");
			htmlContent.append("<p>");
			htmlContent.append("Host:");
			htmlContent.append("<input style=\"width:80px\" type=\"text\" name=\""+EXT_PROXY_HOST+"\" value=\"");
			if (ProxyConfiguration.getExternalProxyHost()!=null)
				htmlContent.append(ProxyConfiguration.getExternalProxyHost());
			htmlContent.append("\">");
			htmlContent.append("&nbsp;Port:");
			htmlContent.append("<input style=\"width:40px\" type=\"text\" name=\""+EXT_PROXY_PORT+"\" value=\"");
			if (ProxyConfiguration.getExternalProxyPort()!=0)
				htmlContent.append(ProxyConfiguration.getExternalProxyPort());
			htmlContent.append("\">");
			htmlContent.append("</p>");
			
			htmlContent.append("<p class=\"dashed\">");
			htmlContent.append("<h2>Proxy Client Restriction</h2>");
			htmlContent.append("Please enter an IP address which is the sole IP address which you will be using to access the Webstretch proxy. No other IP addresses will then be allowed. Be carefull not to lock yourself out!<br/><br/>");
			htmlContent.append("<input type=\"text\" name=\""+IP_RESTRICTION+"\" value=\""+ProxyConfiguration.getRestrictIPAddress()+"\">");
			htmlContent.append("</p>");
			
			
			htmlContent.append("<p><input type=\"submit\" value=\"Apply Changes\"></p>");
			if (changesApplied)
				htmlContent.append("<p><font color=\"red\">Changes Applied!</font></p>");
			htmlContent.append("</form>");
			htmlContent.append(getEndOfHTML());
			
			{
				// send header details		
				String headerContent=getHeaderResponse(htmlContent.toString().getBytes().length);
				out.write(headerContent.getBytes());
			}
			this.closeHeader(out);
			
			// send body
			out.write(htmlContent.toString().getBytes());
	}

}
