package net.sourceforge.webstretch.response;

import java.io.*;
import java.net.*;

/**
 * @author Simon Shanks
 */
public class ErrorResponse extends AbstractResponse {
	
	static final int SERVER_ERROR=500;
	static final int FILE_NOT_FOUND=404;
	
	private int errorCode;
	private String errorMessage;
	
	public ErrorResponse(int errorCode){
		this.errorCode=errorCode;
	}
	
	public ErrorResponse(int errorCode,String message){
		this.errorCode=errorCode;
		this.errorMessage=message;
	}
	
	public ErrorResponse(Exception e){
		this.errorMessage=e.getMessage();
		if (e instanceof UnknownHostException){
			this.errorCode=FILE_NOT_FOUND;
			return;
		}
		if (e instanceof FileNotFoundException){
			this.errorCode=FILE_NOT_FOUND;
			return;
		}
		this.errorCode=SERVER_ERROR;
	}

	/* (non-Javadoc)
	 * @see net.sourceforge.webstretch.server.ProxyResponse#sendResultsToClient(java.io.BufferedOutputStream)
	 */
	public void sendResultsToClient(BufferedOutputStream out)
		throws Exception {
		StringBuffer errorResponse=new StringBuffer("HTTP/1.0 ");
		errorResponse.append(errorCode);
		if (errorMessage!=null){
			errorResponse.append(" ");
			errorResponse.append(errorMessage);
		}			
		errorResponse.append(CRLF);

		out.write(errorResponse.toString().getBytes());
		closeHeader(out);
	}

}
