package net.sourceforge.webstretch.response;

import java.io.BufferedOutputStream;
import java.io.FileNotFoundException;
import java.net.UnknownHostException;

import net.sourceforge.webstretch.request.RequestBody;
import net.sourceforge.webstretch.request.RequestLine;

/**
 * Permits a user to access the robots.txt file of a web server (if present) 
 * and navigate to the areas mentioned.
 * @author Simon Shanks
 */
public class HiddenResponse extends AbstractHTMLResponse {
	
	private static String INPUT_HOST_NAME="hostname";
	private static String ROBOTS_FILE="robots.txt";
	private static String DEFAULT_HTTP_METHOD="HTTP/1.0";
	
	private boolean formSubmitted=false;
	private String robotsHost="";
	private RequestLine request=null;
	
	/**
	 * 
	 * @param body The body containing form input which may be provided by the forms POST method
	 */
	public HiddenResponse(byte[] body){
		if (body==null){
			// form on page hasnt been submitted
			formSubmitted=false;
			return;	
		}	
		RequestBody requestBody=new RequestBody(body);
		{
			// test whether to get robots.txt file
			robotsHost=requestBody.getParameterValue(INPUT_HOST_NAME);
			if (robotsHost!=null && robotsHost.length()>1){
				formSubmitted=true;
				// get host to lookup
				StringBuffer requestLine=new StringBuffer("GET ");
				requestLine.append(robotsHost);
				if (robotsHost.endsWith("/"))
					robotsHost=robotsHost.substring(0,robotsHost.length()-1);
				requestLine.append("/");
				requestLine.append(ROBOTS_FILE);
				requestLine.append(" ");
				requestLine.append(DEFAULT_HTTP_METHOD);
				request=new RequestLine(requestLine.toString());
			}
		}
	}

	/* (non-Javadoc)
	 * @see net.sourceforge.webstretch.response.ProxyResponse#sendResultsToClient(java.io.BufferedOutputStream)
	 */
	public void sendResultsToClient(BufferedOutputStream out) throws Exception{
		StringBuffer htmlContent = new StringBuffer();
		htmlContent.append(getStartOfHTML("WebStretch Hidden File Finder"));
		
		htmlContent.append("<table class=\"headertable\"><tr>");
		htmlContent.append("<td class=\"header\" align=\"center\">");
		htmlContent.append("<a href=\"http://webstretch/request_action\">Request Editor</a></td><td class=\"header\" align=\"center\">");
		htmlContent.append("Find Hidden Files");
		htmlContent.append("</td><td style=\"background:white;\"></td></tr></table>");
		
		htmlContent.append("<h2>Request File</h2>");
		
		htmlContent.append("<p>In order to stop private files from being indexed on search engines, some sites use robots.txt to list these private files.");
		htmlContent.append("&nbsp;This page can retrieve that information</p>");
		htmlContent.append("<form action=\"/hidden_action\" method=\"post\">");
		htmlContent.append("Enter web server address (e.g. http://www.google.com) :&nbsp;");
		htmlContent.append("<br/><input type=\"text\" name=\""+INPUT_HOST_NAME+"\" value=\""+robotsHost+"\">");
		htmlContent.append("<br/><input type=\"submit\" name=\"Submit\">");
		htmlContent.append("</form>");
		
		if (formSubmitted==true){
			htmlContent.append("<h2>Results</h2>");
			// get robots.txt file
			try{
				ExternalResponse externalResponse=new ExternalResponse(request);
				ResponsePayload responsePayload=externalResponse.getExternal();				
				String fileContents=responsePayload.getResponseContents();
				if (fileContents==null || fileContents.length()==0)
					throw new FileNotFoundException();
				htmlContent.append("<pre>");
				htmlContent.append(formatRobotsFile(fileContents));
				htmlContent.append("</pre>");
			} catch (UnknownHostException ex){
				htmlContent.append("<p>No file can be found.<br/>Please check host entered is in the correct format and a connection is available.<p>");
			} catch (Exception e){
				htmlContent.append("<p>No file can be found.<p>");
			}
		}
		
		htmlContent.append(getEndOfHTML());
		{
			// send header details		
			String headerContent=getHeaderResponse(htmlContent.toString().getBytes().length);
			out.write(headerContent.getBytes());
		}
		this.closeHeader(out);
			
		// send body
		out.write(htmlContent.toString().getBytes());
	}
	
	/**
	 * Takes a robots.txt file contents, and inserts anchor links around any
	 * area mentioned
	 * @param fileContents Original robots.txt file
	 * @return HTML formatted contents with links
	 */
	private String formatRobotsFile(String fileContents){
		final String startLinkTagBegin="<a href=\""+robotsHost+"";
		final String startLinkTagEnd="\">";
		final String endLinkTag="</a>";
		boolean windowsFile=false;
		int startPositionOfSearch=0;
		int positionOfDisallowedArea=0;
		int startPositionOfLink=0;
		int endPositionOfLink=0;
		// amount of characters that the inserts are moving the search positions
		int offset=0;
		
		if (fileContents.indexOf("\r\n")!=-1)
			windowsFile=true;
		
		StringBuffer formattedFile=new StringBuffer(fileContents);
		// find links in text file
		while(startPositionOfSearch!=-1){
			endPositionOfLink=-1;
			positionOfDisallowedArea=fileContents.indexOf("Disallow:",startPositionOfSearch);
			if (positionOfDisallowedArea!=-1){
				startPositionOfLink=fileContents.indexOf("/",positionOfDisallowedArea);
				if (startPositionOfLink!=-1){
					if (windowsFile){
						endPositionOfLink=fileContents.indexOf("\r",startPositionOfLink);
					}else{
						endPositionOfLink=fileContents.indexOf("\n",startPositionOfLink);
					}
					if (endPositionOfLink==-1)
						endPositionOfLink=fileContents.length();
					String link=fileContents.substring(startPositionOfLink,endPositionOfLink).trim();
					String alink=startLinkTagBegin+link+startLinkTagEnd;
					formattedFile.insert(startPositionOfLink+offset,alink);
					offset+=alink.length();
					formattedFile.insert(endPositionOfLink+offset,endLinkTag);
					offset+=endLinkTag.length();
				}				
			}
			//System.out.println(positionOfDisallowedArea+" "+startPositionOfLink+" "+endPositionOfLink);
			startPositionOfSearch=endPositionOfLink;
		}
		return formattedFile.toString();	
	}
}
