package net.sourceforge.webstretch.response;

import java.io.BufferedOutputStream;
import java.net.URLDecoder;
import net.sourceforge.webstretch.request.RequestHeaders;
import net.sourceforge.webstretch.request.RequestLine;

public class InspectionResponse extends AbstractHTMLResponse {
	
	private static RequestLine requestLine;
	private static RequestHeaders requestHeaders;
	private static byte[] requestBody;
	private boolean containsAlertWord=false;
	
	public InspectionResponse(){
		this.containsAlertWord=false;
	}
	
	public InspectionResponse(RequestLine requestLine,RequestHeaders requestHeaders,byte[] requestBody,boolean containsAlertWord){
		InspectionResponse.requestLine=requestLine;
		InspectionResponse.requestHeaders=requestHeaders;
		InspectionResponse.requestBody=requestBody;
		this.containsAlertWord=containsAlertWord;
	}
	
	/**
	 * Creates the HMTL for the form element representing the HTTP method possibilities
	 * @param method The method that should be selected by default
	 * @return The HTML for the form element
	 */
	private String methodDropDown(String method) {
		boolean foundMethod = false;
		StringBuffer dropDownHTML =
			new StringBuffer("<select name=\"method\">");
		if (0 == method.compareToIgnoreCase("GET")) {
			dropDownHTML.append("<option value=\"GET\" selected>GET</option>");
			foundMethod = true;
		} else {
			dropDownHTML.append("<option value=\"GET\">GET</option>");
		}
		if ((!foundMethod) && (0 == method.compareToIgnoreCase("POST"))) {
			dropDownHTML.append(
				"<option value=\"POST\" selected>POST</option>");
			foundMethod = true;
		} else {
			dropDownHTML.append("<option value=\"POST\">POST</option>");
		}
		if ((!foundMethod) && (0 == method.compareToIgnoreCase("HEAD"))) {
			dropDownHTML.append(
				"<option value=\"HEAD\" selected>HEAD</option>");
			foundMethod = true;
		} else {
			dropDownHTML.append("<option value=\"HEAD\">HEAD</option>");
		}
		if ((!foundMethod) && (0 == method.compareToIgnoreCase("OPTIONS"))) {
			dropDownHTML.append(
				"<option value=\"OPTIONS\" selected>OPTIONS</option>");
			foundMethod = true;
		} else {
			dropDownHTML.append("<option value=\"OPTIONS\">OPTIONS</option>");
		}
		if ((!foundMethod) && (0 == method.compareToIgnoreCase("TRACE"))) {
			dropDownHTML.append(
				"<option value=\"TRACE\" selected>TRACE</option>");
			foundMethod = true;
		} else {
			dropDownHTML.append("<option value=\"TRACE\">TRACE</option>");
		}
		if ((!foundMethod) && (0 == method.compareToIgnoreCase("PUT"))) {
			dropDownHTML.append("<option value=\"PUT\" selected>PUT</option>");
			foundMethod = true;
		} else {
			dropDownHTML.append("<option value=\"PUT\">PUT</option>");
		}
		if ((!foundMethod) && (0 == method.compareToIgnoreCase("DELETE"))) {
			dropDownHTML.append(
				"<option value=\"DELETE\" selected>DELETE</option>");
		} else {
			dropDownHTML.append("<option value=\"DELETE\">DELETE</option>");
		}
		dropDownHTML.append("</select>");
		return dropDownHTML.toString();
	}
	
	/**
	 * Creates the HMTL for the form element representing the HTTP version possibilities
	 * @param method The version that should be selected by default
	 * @return The HTML for the form element
	 */
	private String httpVersionDropDown(String version) {
		StringBuffer dropDownHTML =
			new StringBuffer("<select name=\"version\"><option>&lt;blank&gt;</option>");
		if (0 == version.compareToIgnoreCase("HTTP/1.1")) {
			dropDownHTML.append("<option selected>HTTP/1.1</option>");
		} else {
			dropDownHTML.append("<option>HTTP/1.1</option>");
		}
		if (0 == version.compareToIgnoreCase("HTTP/1.0")) {
			dropDownHTML.append("<option selected>HTTP/1.0</option>");
		} else {
			dropDownHTML.append("<option>HTTP/1.0</option>");
		}
		if (0 == version.compareToIgnoreCase("HTTP/0.9")) {
			dropDownHTML.append("<option selected>HTTP/0.9</option>");
		} else {
			dropDownHTML.append("<option>HTTP/0.9</option>");
		}
		dropDownHTML.append("</select>");
		return dropDownHTML.toString();
	}

	/* (non-Javadoc)
	 * @see net.sourceforge.webstretch.server.ProxyResponse#sendResultsToClient(java.io.BufferedOutputStream)
	 */
	public void sendResultsToClient(BufferedOutputStream out)
		throws Exception {
		StringBuffer htmlContent = new StringBuffer();
		htmlContent.append(getStartOfHTML("WebStretch Inspection"));

		htmlContent.append("<table class=\"headertable\"><tr>");
		htmlContent.append("<td class=\"header\" align=\"center\">");
		htmlContent.append("Request Editor</td><td class=\"header\" align=\"center\">");
		htmlContent.append("<a href=\"http://webstretch/hidden_action\">Find Hidden Files</a>");
		htmlContent.append("</td><td style=\"background:white;\"></td></tr></table>");
		
		htmlContent.append("<h2>Previous Request Inpection</h2>");
					
		if (requestLine==null){
			htmlContent.append("<p>In order to start inspecting requests, please enter a URL of another web site into your browser.</p>");
			htmlContent.append(getEndOfHTML());
			{
				// send header details		
				String headerContent=getHeaderResponse(htmlContent.toString().getBytes().length);
				out.write(headerContent.getBytes());
			}
			this.closeHeader(out);
		
			// send body
			out.write(htmlContent.toString().getBytes());
			return;
		}
					
		if (this.containsAlertWord)
			htmlContent.append("<p align=\"center\"><font color=\"red\">Request Contains Alert Word!</font></p>");
		htmlContent.append("<table width=\"100%\"><form action=\"http://"+requestLine.getHost()+":"+requestLine.getPort()+"/proxyprocess\" method=\"post\"><tr>");
		htmlContent.append("<input type=\"hidden\" name=\"host\" value=\"");
		htmlContent.append(requestLine.getHost());
		htmlContent.append("\" />");
		htmlContent.append("<td width=\"20%\">Method</td><td>");
		htmlContent.append(this.methodDropDown(requestLine.getMethod()));
		htmlContent.append("</td></tr>");
		htmlContent.append(
			"<td width=\"20%\">URI</td><td><input type=\"text\" name=\"uri\" value=\"");
		htmlContent.append(requestLine.getUri());
		htmlContent.append("\" /></td></tr>");
		htmlContent.append(
			"<td width=\"20%\">Port</td><td><input type=\"text\" name=\"port\" value=\"");
		htmlContent.append(requestLine.getPort());
		htmlContent.append("\" /></td></tr>");
		htmlContent.append(
			"<td width=\"20%\">HTTP Version</td><td>");
		htmlContent.append(this.httpVersionDropDown(requestLine.getHttpVersion()));
		htmlContent.append("</td></tr>");
		htmlContent.append("</tr></table>");

		htmlContent.append("<p>Parameters</p>");
		htmlContent.append("<table width=\"100%\"><tr>");
		String[] parameterNames = requestLine.getParameterNames();
		String parameterName = null;
		String parameterValue = null;
		for (int i = 0; i < parameterNames.length; i++) {
			parameterName = parameterNames[i];
			parameterValue = requestLine.getParameterValue(parameterName);
			htmlContent.append("\n\r<td width=\"20%\">");
			htmlContent.append(parameterName);
			htmlContent.append("</td><td><input type=\"text\" name=\"param");
			htmlContent.append(parameterName);
			htmlContent.append("\" value=\"");
			if (parameterValue!=null)
				htmlContent.append(parameterValue);
			htmlContent.append("\" /></td></tr>\n\r");
		}
		htmlContent.append("</tr></table>");

		htmlContent.append("<p>Headers</p>");
		htmlContent.append("<table width=\"100%\"><tr>");

		String[] headerNames = requestHeaders.getHeaderNames();
		String headerName = null;
		for (int i = 0; i < headerNames.length; i++) {
			headerName = headerNames[i];
			htmlContent.append("\n\r<td width=\"20%\">");
			htmlContent.append(headerName);
			htmlContent.append("</td><td><input type=\"text\" name=\"head");
			htmlContent.append(headerName);
			htmlContent.append("\" value=\"");
			htmlContent.append(requestHeaders.getHeaderValue(headerName));
			htmlContent.append("\" /></td></tr>\n\r");
		}
		htmlContent.append("</tr></table>");

		htmlContent.append(
			"<p>Body</p><table><tr><td><textarea rows=\"5\" name=\"body\">");
		if (requestBody != null) {
			String body = new String(requestBody);
			htmlContent.append(URLDecoder.decode(body));
		}
		htmlContent.append("</textarea></td></tr></table>");

		htmlContent.append(
			"<p><input type=\"submit\" value=\"Submit Request\"></p></form>");
			
		htmlContent.append(getEndOfHTML());
		{
			// send header details		
			String headerContent=getHeaderResponse(htmlContent.toString().getBytes().length);
			out.write(headerContent.getBytes());
		}
		this.closeHeader(out);
		
		// send body
		out.write(htmlContent.toString().getBytes());
	}

}
