package net.sourceforge.webstretch.response;

import java.util.ArrayList;

/**
 * @author Simon Shanks
 */
public class ResponsePayload {
	
	private static final String LINE_SEPERATOR="\r\n";
	
	byte[] fullResponseContents;
	private String fullTextResponse="";
	
	private static final String COMMENT_START_TAG="<!--";
	private static final String COMMENT_END_TAG="-->";
	private static final int COMMENT_START_TAG_LENGTH=COMMENT_START_TAG.length();
	
	protected ResponsePayload(){
	}
	
	protected String getResponseLine(){
		int endOfLine=fullTextResponse.indexOf(LINE_SEPERATOR);
		if (endOfLine==-1)
			return null;
		return fullTextResponse.substring(0,endOfLine);
	}
	
	protected String getResponseContents(){
		int startOfContents=fullTextResponse.indexOf(LINE_SEPERATOR+LINE_SEPERATOR);
		if (startOfContents==-1)
			return null;
		return fullTextResponse.substring(startOfContents+4,fullTextResponse.length());
	}
	
	protected Object[] getComments(){
		ArrayList comments=new ArrayList();
		if (fullResponseContents==null || fullResponseContents.length==0)
			return new String[0];
		String fullResponse=this.toString();
		if (fullResponse==null || fullResponse.length()==0)
			return new String[0];
		int startOfComment=0;
		int endOfComment=0;
		do{
			startOfComment=fullResponse.indexOf(COMMENT_START_TAG,endOfComment);
			if (startOfComment>0){
				startOfComment+=COMMENT_START_TAG_LENGTH;
				endOfComment=fullResponse.indexOf(COMMENT_END_TAG,startOfComment);
				if (endOfComment>startOfComment)
					comments.add(fullResponse.substring(startOfComment,endOfComment));									
			}		
		} while (startOfComment>0);	
		if (comments==null || comments.isEmpty())
			return new String[0];
		return comments.toArray();
	}
	
	byte[] getResponseData(){
		//return fullTextResponse.getBytes();
		return fullResponseContents;
	}
	
	protected void addResponseData(byte[] buffer,int bytesRead){
		// add to byte array
		int currentEndPointer=0;
		if (fullResponseContents!=null)	
			currentEndPointer=fullResponseContents.length;
		byte [] tempFullResponseContents=new byte[currentEndPointer+bytesRead];
		// copy in old contents
		for (int i=0;i<currentEndPointer;i++){
			tempFullResponseContents[i]=fullResponseContents[i];
		}
		// add new contents
		for (int i=0;i<bytesRead;i++,currentEndPointer++){
			tempFullResponseContents[currentEndPointer]=buffer[i];
		}
		fullResponseContents=tempFullResponseContents;
		
		// add to text 
		StringBuffer currentResponse=new StringBuffer(this.fullTextResponse);
		// add new contents
		for (int i=0;i<bytesRead;i++){
			currentResponse.append((char)buffer[i]);
		}
		fullTextResponse=currentResponse.toString();
	}
	
	public String toString(){
		/*StringBuffer result=new StringBuffer(fullResponseContents.length);
		for (int i=0;i<fullResponseContents.length;i++){
			result.append((char)fullResponseContents[i]);
		}
		return result.toString();*/
		return fullTextResponse;
	}
}
