package net.sourceforge.webstretch.server;

import java.util.*;

import net.sourceforge.webstretch.request.*;

/**
 * Records a list of words, in which the user wishes to be alerted to their
 * use, as part of a web request
 * @author Simon Shanks
 */
public class AlertWords {
		
	private static ArrayList words=new ArrayList();
	private static final String WORD_SEPERATOR=",";
	
	public static boolean hasAlertWords=false;
	
	/**
	 * Resets the system so that it doesnt have a record of any alert words
	 */
	public static void reset(){
		words.clear();
		hasAlertWords=false;
	}
	
	/**
	 * Sets the list of word(s) which the system should be configured to look
	 * for in requests
	 * @param alertWords A single word, or comma seperated list of words
	 */
	public static void setWords(String alertWords){
		AlertWords.reset();
		if (alertWords==null || alertWords.length()==0)
			return;
		StringTokenizer wordTokenizer=new StringTokenizer(alertWords,WORD_SEPERATOR);
		while (wordTokenizer.hasMoreTokens()){
			String word=wordTokenizer.nextToken();
			words.add(word);
		}
		words.trimToSize();
		hasAlertWords=true;		
	}
	
	/**
	 * Gets a list of alert words which the system is currently configured to look
	 * for in requests
	 * @return List of alert words, seperated by commas
	 */
	public static String getWords(){
		if (hasAlertWords==false || words==null || words.isEmpty())
			return "";
		StringBuffer allAlertWords=new StringBuffer();
		Iterator wordIterator=words.iterator();
		int wordCount=0;
		while (wordIterator.hasNext()){
			++wordCount;
			if (wordCount>1)
				allAlertWords.append(WORD_SEPERATOR);
			allAlertWords.append((String)wordIterator.next());
		}
		return allAlertWords.toString();
	}
	

	/**
	 * Tests whether a string contains an alert word
	 * @param text The string which is to be tested
	 * @return true is the string input contains an alert word
	 */
	public static boolean containsAlertWord(String text){
		if (words==null || words.isEmpty() || text==null || text.length()==0)
			return false;
		Iterator wordIterator=words.iterator();
		while (wordIterator.hasNext()){
			String currentAlertWord=(String)wordIterator.next();
			if (-1!=text.toLowerCase().indexOf(currentAlertWord.toLowerCase()))
				return true;
		}
		return false;
	}
	
	/**
	 * @see #containsAlertWord(java.lang.String)
	 */
	public static boolean containsAlertWord(byte[] text){
		if (text==null || text.length==0)
			return false;
		return containsAlertWord(RequestBody.convertBodyToString(text));
	}

}
