package net.sourceforge.webstretch.server;

import java.net.*;
import java.io.*;

import net.sourceforge.webstretch.response.*;

/**
 * @author Simon Shanks
 */
public class ProcessRequest extends Thread {

	Socket socket=null;
	
	/**
	 * Takes care of a single resource request, taking input/output from the
	 * socket connection
	 * @param socket The socket to which the client is connected
	 */
	public ProcessRequest(Socket socket) throws UnauthorizedClientException{
		String restrictToIPAddress=ProxyConfiguration.getRestrictIPAddress();
		if (restrictToIPAddress!=null && restrictToIPAddress.length()>0){
			InetAddress  clientInetAddress=socket.getInetAddress();
			String clientIPAddress=clientInetAddress.getHostAddress();
			if (!clientIPAddress.equals(restrictToIPAddress))
				throw new UnauthorizedClientException("Unauthorized access attempt by "+clientIPAddress);
		}		
		this.socket=socket;
	}
	
	/**
	 * Takes care of the input/output details of the client request
	 */
	public void run(){
		if (!ProxyServer.serverOperational)
			return;
		BufferedInputStream in = null;
		BufferedOutputStream out = null;
		try{
			try {
				// Client connection now created
				in = new BufferedInputStream(socket.getInputStream());
				out = new BufferedOutputStream(socket.getOutputStream());
				// deal with request send from  browser
				AbstractResponse proxyResponse=ResponseFactory.getResponse(in);
				// send reply back to the browser
				proxyResponse.sendResultsToClient(out);
			}catch (UnknownHostException uhe){
				ErrorResponse errorResponse=new ErrorResponse(uhe);
				errorResponse.sendResultsToClient(out);
			}catch (FileNotFoundException e) {
				ErrorResponse errorResponse=new ErrorResponse(e);
				errorResponse.sendResultsToClient(out);
			}catch (ShutdownException sde) {
				ShutdownResponse shutdownResponse=new ShutdownResponse();
				shutdownResponse.sendResultsToClient(out);
				throw sde;
			}catch (Exception e) {
				System.err.println(e.getMessage());
				e.printStackTrace();
				throw e;
			} finally {
				try {
					if (in != null)
						in.close();
					if (out != null)
						out.close();
					if (socket != null)
						socket.close();
				} catch (IOException e) {
					e.printStackTrace();
					throw e;
				}
			}
		} catch (Exception e){
			// shutdown proxy
			ProxyServer.serverOperational=false;
		}
	}

}
