package net.sourceforge.webstretch.server;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * Holds configuration data for the proxy server
 * @author Simon Shanks
 */
public class ProxyConfiguration {
	
	private static final String PROPERTY_FILE = "server.properties";
	
	public static final String RELATIVE_WEB_ROOT = "webroot";
	public static final String PROXY_HOST = "webstretch";
	
	public static int PROXY_PORT=8080;
	static int MAXIMUM_THREADS=3;
	
	private static String installationPath;
	private static String webRootPath;
	private static Properties properties = new Properties();
	
	private static String externalProxyHost;
	private static int externalProxyPort;
	private static String restrictIPAddress;
	
	static {
		ProxyConfiguration.setInstallationPath(System.getProperty("user.dir"));
		try {
			// read properties file
			properties.load(new FileInputStream(PROPERTY_FILE));

			// get various properties from properties file
			externalProxyHost = properties.getProperty("externalproxy.host");
			restrictIPAddress = properties.getProperty("restrictToIP.address");
			String externalProxyPortProperty = properties.getProperty("externalproxy.port");
			if ((externalProxyPortProperty != null)
				&& (externalProxyPortProperty.length() != 0))
				externalProxyPort = Integer.parseInt(externalProxyPortProperty);
			String proxyPortProperty = properties.getProperty("proxy.port");
			if ((proxyPortProperty != null)
				&& (proxyPortProperty.length() != 0))
				PROXY_PORT = Integer.parseInt(proxyPortProperty);
			String maximumThreadsProperty = properties.getProperty("maximum.threads");
			if ((maximumThreadsProperty != null)
				&& (maximumThreadsProperty.length() != 0))
				MAXIMUM_THREADS = Integer.parseInt(maximumThreadsProperty);		
				
		} catch (IOException e) {
			System.err.println("Cannot load input file " + PROPERTY_FILE);
			System.err.println("Using default settings");
		}
	}

	/**
	 * Gets the existing proxy that the web request must use to get the resource
	 * @return null if property not in configuration file
	 */
	public static String getExternalProxyHost() {
		return externalProxyHost;
	}

	/**
	 * @return The port of an external proxy server. Returns 0 if none defined.
	 */
	public static int getExternalProxyPort() {
		return externalProxyPort;
	}

	/**
	 * @param host The host of an external proxy server. Returns null if none defined.
	 */
	public static void setExternalProxyHost(String host) {
		externalProxyHost = host;
	}

	/**
	 * @param Sets the port of an external proxy server. Set to 0 to reset.
	 */
	public static void setExternalProxyPort(int port) {
		externalProxyPort = port;
	}
	
	/**
	 * Sets the installation path of the proxy server
	 * @param path The installation path of the proxy server
	 */
	public static void setInstallationPath(String path) {
		installationPath = path;
		// make changes to webRootPath as this will be effected
		if (installationPath.endsWith(File.separator)) {
			webRootPath = installationPath + RELATIVE_WEB_ROOT;
		} else {
			webRootPath = installationPath + File.separator + RELATIVE_WEB_ROOT;
		}
	}

	/**
	 * 
	 * @return true if there is an external proxy in use.
	 */
	public static boolean isProxyDefined() {
		return (
			(externalProxyHost != null)
				&& (externalProxyHost.length() != 0)
				&& (externalProxyPort != 0));
	}
	
	/**
	 * @return
	 */
	public static String getRestrictIPAddress() {
		if (restrictIPAddress==null)
		 	return "";
		else
			return restrictIPAddress;
	}

	/**
	 * @param string
	 */
	public static void setRestrictIPAddress(String string) {
		restrictIPAddress = string;
	}

}
