package net.sourceforge.webstretch.server;

import java.io.*;
import java.net.*;

/**
 * See 	<br>
 * 		RFC 1945 (HTTP 1.0)<br>
 * 		RFC 2616 (HTTP 1.1)<br>
 * 		RFC 2965 (cookie state management)<br>
 * 		RFC 2046 (MIME)<br>
 * 		RFC 2518 (WebDAV)
 * @author Simon Shanks
 */
public class ProxyServer {
	
	static boolean serverOperational=true;

	/**
	 * Creates a proxy and deals with client requests, creating a new
	 * thread for each request.
	 * @param proxyPort
	 */
	public ProxyServer(int proxyPort) {
		ServerSocket srv = null;
		try {
			srv=new ServerSocket(proxyPort);
			// the timeout period will usually take effect after a shutdown has occured
			srv.setSoTimeout(300);
			// keep looping while not shutting down - note the blocking
			// accept method in the loop will require a timeout period
			while (serverOperational) {
				try {
					if (Thread.activeCount()>ProxyConfiguration.MAXIMUM_THREADS)
						Thread.yield();
					ProcessRequest processRequest=new ProcessRequest(srv.accept());
					processRequest.start();
				} catch (InterruptedIOException e){
			    } catch (UnauthorizedClientException e){System.err.println(e.getMessage());}
			}
		} catch (BindException e) {
			System.err.println("Port already in use, please choose another");
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				if (srv != null)
					srv.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	/**
	 * Enables command line execution of proxy server
	 * @param args	Command line arguements are currently not used.
	 */
	public static void main(String[] args) {
		String IPAddress=null;
		try{
			IPAddress=InetAddress.getLocalHost().getHostAddress();
		} catch (Exception e){
			System.err.println("ERROR : This machine doesnt have an IP address");
			System.err.println("Try connecting to a network or installing a loopback adapter");
			System.exit(-1);
		}
				
		System.out.println("**     Starting webstretch proxy          ***");
		System.out.println("  Proxy web page URL is http://webstretch");
		System.out.println("*********************************************");
		System.out.println("* Ensure browser has the following settings");
		System.out.println("* Proxy IP address = "+IPAddress);
		System.out.println("* Proxy port       = "+ProxyConfiguration.PROXY_PORT);
		System.out.println("*********************************************");
		ProxyServer proxyServer=new ProxyServer(ProxyConfiguration.PROXY_PORT);
		System.out.println("\n** Shutting down webstretch proxy         ...");
	}
}