package net.sourceforge.webstretch.server;

import java.io.*;

import net.sourceforge.webstretch.request.*;
import net.sourceforge.webstretch.response.*;

/**
 * Determines the type of a response the request should generate
 * @author Simon Shanks
 */
public class ResponseFactory {

	public static AbstractResponse getResponse(BufferedInputStream in)
		throws ShutdownException {
		RequestLine requestLine = null;
		RequestHeaders requestHeaders = null;
		byte[] requestBody = null;
		boolean containsAlertWord=false;
		// extract info from request
		try {
			{
				RequestPayload requestPayload = new RequestPayload(in);
				requestLine = new RequestLine(requestPayload.getRequestLine());
				requestHeaders = requestPayload.getRequestHeaders();
				requestBody = requestPayload.getBody();
				containsAlertWord=requestPayload.containsAlertWord();
			}
			
			if (requestLine.isRequestLocal()) {
				// request for file in webstretch system
				// TODO need to make this more dynamic - poor at the moment, but works
				if (requestLine.getUri().equalsIgnoreCase("/configserver_action")) 
					return new ConfigServerResponse(requestBody);
				else if (requestLine.getUri().equalsIgnoreCase("/configrequest_action")) 
					return new ConfigRequestResponse(requestBody);
				else if (requestLine.getUri().equalsIgnoreCase("/help_action")) 
						return new HelpResponse();
					else if (requestLine.getUri().equalsIgnoreCase("/statistics_action")) 
							return new StatisticsResponse();
						else if (requestLine.getUri().equalsIgnoreCase("/request_action")) 
								return new InspectionResponse();	
							else if (requestLine.getUri().equalsIgnoreCase("/hidden_action")) 
									return new HiddenResponse(requestBody);	
								else if (requestLine.getUri().equalsIgnoreCase("/shutdown_action"))
									throw new ShutdownException(); 
									else
										return new IndexResponse();
			} else if (
				// request for external web resource from inspection page
				requestLine.getUri().equalsIgnoreCase("/proxyprocess")) {
				return new ExternalResponse(requestBody);
			} else {
				// need to decide whether this request needs to result in an inspection page
				InspectionDecision inspectionDecision=new InspectionDecision(requestLine,containsAlertWord);
				if (inspectionDecision.isResourceToInspect()){
					// request for request inspection page
					return new InspectionResponse(
						requestLine,
						requestHeaders,
						requestBody,containsAlertWord);					
				}
				else{
					// request for external web resource
					return new ExternalResponse(
						requestLine,
						requestHeaders,
						requestBody);					
				}
			}
		} catch (ShutdownException sde){
			throw sde;
		} catch (FileNotFoundException fnfe) {
			return new ErrorResponse(fnfe);
		} catch (IOException ioe) {
			ioe.printStackTrace();
			return new ErrorResponse(ioe);
		}
	}

}
