package net.sourceforge.webstretch.server;

import java.util.HashMap;

/**
 * Keeps tack of web browsing statistics
 * @author Simon Shanks
 */
public class Statistics {

	private static long totalBytesReceived=0;
	private static HashMap bytesReceivedByHosts=new HashMap();

	/**
	 * Adds a record of how many bytes the current host has sent. If data was previously
	 * sent by this host, it is added to its current total
	 * @param host The host which has sent the data
	 * @param bytesCountToAdd The amount of data the host has recently sent
	 */
	public synchronized static void addToTotalBytesReceived(String host,long bytesCountToAdd){
		totalBytesReceived+=bytesCountToAdd;
		Object countObject=bytesReceivedByHosts.get(host);
		if (null==countObject)
			bytesReceivedByHosts.put(host,new Long(bytesCountToAdd));
		else{
			long currentCount=((Long)countObject).longValue();
			currentCount+=bytesCountToAdd;
			bytesReceivedByHosts.put(host,new Long(currentCount));
		}	
	}
	
	/**
	 * Gets all hosts which have responded with information, with totals by host
	 * @return Map, using string host as key. Returns an empty HashMap if no recordings made
	 */
	public synchronized static HashMap getTotalsByHost(){
		return bytesReceivedByHosts;
	}
	
	/**
	 * Gets the total bytes received all hosts
	 * @return Total number of bytes received
	 */
	public static long getToTotalBytesReceived(){
		return totalBytesReceived;
	}
}